def PrintBytes(size: int) -> str:
    if size > 1024:
        size = round(size / 1024, 2);
        if(size > 1024):
            size = round(size / 1024, 4);
            if(size > 1024):
                size = round(size / 1024, 8);
                if(size > 1024):
                    size = round(size / 1024, 16);
                    return f"{size} TB";
                else:
                    return f"{size} GB";
            else:
                return f"{size} MB";
        else:
            return f"{size} KB";
   
    return f"{size} B";

def CheckStringInFile(file_path, target_string):
    try:
        with open(file_path, 'r') as file:
            for line in file:
                if target_string in line:
                    return True;
        return False;
    except FileNotFoundError:
        return False;

def CleanArrayFromEmptyString(array: list) -> list:
    return [string for string in array if string != ""];

def ReadFile(file_path: str) -> list:
    with open(file_path, 'r') as file:
        lines = file.readlines();
    return lines;


class Result:
    def __init__(self, status: bool, message: str = "", payload: list = []):
        self.__Status = status;
        self.__Message = message;
        self.__Payload = payload;

    def IsSuccess(self) -> bool:
        return self.__Status;

    def GetMessage(self) -> str:
        return self.__Message;


from datetime import datetime, timedelta;

now = datetime.now();

def CalculateTime(start: datetime):
    return datetime.now() - start;

def TimeToString(time: datetime) -> str:
    seconds = time.total_seconds();
    minutes = seconds // 60;
    hours = minutes // 60;
    days = hours // 24;
    return f"{round(days)} Days = {round(hours)} Hours = {round(minutes)} Minutes = {round(seconds, 2)} Seconds";

def AddTime(Hours: int) -> str:
    return (datetime.now() + timedelta(hours=Hours)).strftime('%d-%m-%Y_%H:%M:%S');


from lxml import etree;
from AlmaxClasses.Result import *;

def XML_ValidateXSD(xml_path: str, xsd_path: str) -> Result:
    with open(xsd_path, 'r') as schema_file:
        schema = etree.XMLSchema(etree.parse(schema_file));
    with open(xml_path, 'r') as xml_file:
        doc = etree.parse(xml_file);
    try:
        schema.assertValid(doc);
        return Result(True, "XML document is valid");
    except etree.DocumentInvalid as e:
        return Result(False, f"XML document is invalid: {e}");


from AlmaxClasses.Result import *;

from __future__ import annotations;

class Example:
    def __init__(self, moinvId: str):
        self.__Id = f"MOINV_{moinvId.strip("0")}";
        self.__Observations = [];
   
    @staticmethod
    def fromLine(line: list) -> Example:
        return Example(f"{line[3:5]}");
   
    @property
    def Id(self) -> str:
        return self.__Id;

    @property
    def Observations(self) -> list:
        return self.__Observations;

    @Observations.setter
    def Observations(self, value):
        self.__Observations = value;

    @property
    def LastObsID(self) -> str:
        return (len(self.__Observations) - 1);

    @classmethod
    def addObservation(self, line: str):
        newObs = f"{int(line[3:5])}";
        self.__Observations.append(newObs);

    @classmethod
    def CheckIfIdIsSame(self, line: str) -> bool:
        newCubo = Example.fromLine(line);
        return self.Id == newCubo.Id;


import AlmaxUtils.Generic as Generic;
import os;

from reportlab.lib.pagesizes import A4;
from reportlab.lib import colors;
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle;
from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer, Table, TableStyle;

def GeneratePdf(client_info, orders):
    try:
        now = Generic.now();
        now_month = now.month if now.month > 9 else f"0{now.month}";
        now_day = now.day if now.day > 9 else f"0{now.day}";
        now_hour = now.hour if now.hour > 9 else f"0{now.hour}";
        now_minute = now.minute if now.minute > 9 else f"0{now.minute}";
        now_second = now.second if now.second > 9 else f"0{now.second}";

        file_path = f"{now.year}/{now_month}";
        if not os.path.exists(file_path):
            os.makedirs(file_path);
        file_path = f"{file_path}/{now_day}_{now_hour}{now_minute}{now_second}.pdf";
        file_path = "test.pdf";

        size_societyName = 25;
        margin = 10;
        doc = SimpleDocTemplate(
            file_path, 
            pagesize=A4, 
            leftMargin=margin, 
            rightMargin=margin, 
            topMargin=margin, 
            bottomMargin=margin
        );

        content = [];
        styles = getSampleStyleSheet();

        header_table_data = [
            [
                Paragraph(
                    f"<font color='red' size={size_societyName}>M</font><font color='black' size={size_societyName}>A</font> SRLS", 
                    ParagraphStyle(
                        name=any,
                        parent=styles['Normal'],
                        fontName='Times-Roman',  # Cambia con il tuo font preferito
                        fontSize=12
                    )
                ), 
                "Preventivo"
            ],
            [
                "", 
                ""
            ],
            [
                Paragraph(
                    f"Via Cefalonia 24, Brescia (BS)", 
                    ParagraphStyle(
                        name=any,
                        parent=styles['Normal'],
                        fontName='Times-Roman',
                    )
                ), 
                f"Spettabile {client_info['name']}"
            ],
            [
                Paragraph(
                    f"P.IVA 04033430986", 
                    ParagraphStyle(
                        name=any,
                        parent=styles['Normal'],
                        fontName='Times-Roman',
                    )
                ), 
                ""
            ]
        ];
            
        header_table_style = TableStyle([
            # startcol, startrow - endcol, endrow
            ('FONT',(0, 0), (1, 0), 'Times-Bold'),
            ('SIZE',(0, 0), (1, 0), 15),
            ('ALIGN', (0, 0), (-1, -1), 'CENTER'),
            ('VALIGN', (0, 0), (-1, -1), 'MIDDLE'),

            ('TEXTCOLOR', (0, 0), (1, 0), colors.red)
        ]);
        
        header_table = Table(header_table_data, colWidths=[doc.width / 2 - 10, doc.width / 2 - 10]);
        header_table.setStyle(header_table_style);
        content.append(header_table);

        content.append(Spacer(1, 10))  # Aggiungi spaziatura

        body_table_data = [
            [
                "Descrizione", 
                "Quantità",
                "Prezzo",
                "Totale (no IVA)"
            ]
        ];
        for order in orders:
            body_table_data.append(ToParagraph_ForTable(order.Description, order.Quantity, order.Price, order.Total));

        body_table_style = TableStyle([
            ('BACKGROUND', (0, 0), (-1, 0), colors.whitesmoke),
            ('ALIGN', (0, 0), (-1, -1), 'CENTER'),
            ('VALIGN', (0, 0), (-1, -1), 'MIDDLE'),
            ('GRID', (0, 0), (-1, -1), 1, colors.black),

            ('TEXTCOLOR', (0, 0), (-1, -1), colors.red),
            ('FONTNAME', (0, 0), (-1, 0), 'Helvetica-Bold')
        ]);
        body_table = Table(
            body_table_data,
            colWidths=[370,48,60,80]
        );
        body_table.setStyle(body_table_style);
        content.append(body_table);

        content.append(Spacer(1, 12))  # Aggiungi spaziatura
        content.append(Paragraph("Il tuo testo qui", styles['Normal']))

        # Build the PDF
        doc.build(content);

        print(f'PDF generated successfully at: {file_path}');
    except Exception as e:
        print(f'Error generating PDF: {e}');


def ToParagraph_ForTable(Description, Quantity, Price, Total):
    style = ParagraphStyle(
        name="Centered",
        parent=getSampleStyleSheet()['Normal'],
        fontName='Times-Roman',
        alignment=1 # 0=left, 1=center, 2=right
    );
    return [
        Paragraph(
            f"{Description}"
        ),
        Paragraph(
            f"{Quantity}", 
            style
        ),
        Paragraph(
            f"{Price}€", 
            style
        ),
        Paragraph(
            f"{Total}€", 
            style
        )
    ];


import subprocess;

class CustomSubprocess:
    def __init__(self):
        self.__Command = [];
        self.__Istance = None;
   
    def Run(self, command: list):
        self.ChangeCommand(command);
        self.Start();
   
    def ChangeCommand(self, command):
        self.__Command = command;

    def Start(self):
        if self.__Istance is None or self.__Istance.poll() is not None:
            self.__Istance = subprocess.Popen(
                self.__Command,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE
            );
   
    def HasFinished(self):
        return (self.__Istance is not None and self.__Istance.poll() is not None);

    def Stop(self):
        if self.__Istance:
            self.__Istance.terminate();
            self.__Istance = None;



from setuptools import setup, find_packages;

with open("README.md", "r") as fh:
    readMe = fh.read();

with open("requirements.txt") as f:
    required = f.read().splitlines();

with open("version.txt", "r") as fh:
    versionFile = fh.read().strip();

setup(
    name='almax_common',
    version=versionFile,
    description='A common library with some of my implementations',
    long_description=readMe,
    long_description_content_type='text/markdown',
    author='AlMax98',
    author_email='alihaider.maqsood@gmail.com',
    packages=find_packages(),
    package_dir={'': '.'},
    install_requires=required,
);


import threading;
from typing import Callable;
from queue import Queue;

class CustomThread:
    def __init__(self):
        self.__Istance = None;
        self.__Result = Queue();
   
    def Run(self, function: Callable[[str, int], int], par1: str, par2: int):
        self.__Istance = threading.Thread(target=function, args=(par1, par2));
        self.__Istance.start();
   
    def GetResults(self):
        return self.__Result.get();
   
    def HasFinished(self):
        if self.__Istance.is_alive():
            return False;
        else:
            self.__Result.put("ok");
            return True;

    def Stop(self):
        self.__Istance.join(timeout=0);


import tkinter as TK;
from tkinter import ttk;
from tkinter import messagebox;
import sys;

WindowElementsDefaultPaddingWidth = 10;
WindowElementsDefaultPaddingHeight = 10;

def WindowCreateLabel(
    Window: TK.Tk,
    Text: str
) -> TK.Label:
    WindowLabel = TK.Label(
        Window,
        text=Text,
        padx=WindowElementsDefaultPaddingWidth,
        pady=WindowElementsDefaultPaddingHeight,
        anchor=TK.W
    );
    return WindowLabel;

def WindowCreateButton(
    Window: TK.Tk,
    Text: str,
    CommandIndex: int,
    AssignCommand
) -> TK.Button:
    WindowButton = TK.Button(
        Window,
        text = Text,
        command = lambda: AssignCommand(WindowButton, CommandIndex),
        width=WindowElementsDefaultPaddingWidth,
        padx=WindowElementsDefaultPaddingWidth
    );
    return WindowButton;

def WindowCreateRadioButton(
    Window: TK.Tk,
    Options: list,
    WindowRadioButtonSelected: TK.IntVar
):
    radioButtons = [];
    for (optionLabel, optionValue) in Options:
        radioButtons.append(TK.Radiobutton(
            Window,
            text = optionLabel,
            value = optionValue,
            variable=WindowRadioButtonSelected,
            width=WindowElementsDefaultPaddingWidth
        ));
    return radioButtons;

def WindowCreateProgressbar(
   Window: TK.Tk
):
    progressbar = ttk.Progressbar(
        Window,
        mode="determinate",
        orient="horizontal",
        maximum=100,
        length=286
    );
    return progressbar;
     
class ProgressbarProgression:
    def __init__(
        self,
        progressbarRow: ttk.Progressbar,
        progressbarPercRow: TK.Label,
        progressRatio: int = 50
    ):
        self.__Progression = 0;
        self.__ProgressbarRow = progressbarRow;
        self.__ProgressbarPercRow = progressbarPercRow;
        self.__ProgressRatio = progressRatio;
   
    def Update(self):
        self.__Progression += self.__ProgressRatio;
        self.__ProgressbarRow["value"] = self.__Progression;
        self.__ProgressbarRow.update();
        self.__ProgressbarPercRow["text"] = str(round(self.__Progression, 2)) + "%";
        self.__ProgressbarPercRow.update();

    def ChangeProgressionRatio(self, progressionRatio: int):
        self.__ProgressRatio = progressionRatio;

class Window:
    def __init__(
        self,
        programName: str
    ):
        self.__Istance = TK.Tk();
        self.__Istance.resizable(False, False);
        self.__Istance.title(programName);

        self.__IsRunning = False;

        self.__Frames = {};
        self.__ButtonsToDisable = [];
        self.__RadioButtons = [];
   
    def AddFrame(self, frameName, position):
        self.__Frames[frameName] = TK.Frame(self.__Istance);
        self.__Frames[frameName].pack(anchor=position);
   
    def AddLabelToFrame(self, labelName: str, frameName: str, position):
        WindowLabelRow = WindowCreateLabel(self.__Istance, labelName);
        WindowLabelRow.pack(in_=self.__Frames[frameName], side=position);
        return WindowLabelRow;
   
    def AddButtonToFrame(self, buttonName: str, commandIndex: int, commandFunction, frameName: str, position, disableAtSubmit: bool = True, defaultDisabled: bool = False) -> TK.Button:
        WindowButtonRow = WindowCreateButton(self.__Istance, buttonName, commandIndex, commandFunction);
        WindowButtonRow.pack(in_=self.__Frames[frameName], side=position);
        if disableAtSubmit:
            self.__ButtonsToDisable.append(WindowButtonRow);
        if defaultDisabled:
            WindowButtonRow['state'] = "disabled";
            WindowButtonRow['width'] = len(WindowButtonRow['text'])-5;
        return WindowButtonRow;

    def AddRadioButtonToFrame(self, WindowRadioButtonOptions, WindowRadioButtonSelected, frameName, position):
        WindowRadioButtonRow = WindowCreateRadioButton(self.__Istance, WindowRadioButtonOptions, WindowRadioButtonSelected);
        for rb in WindowRadioButtonRow:
            rb.pack(in_=self.__Frames[frameName], side=position);
        self.__RadioButtons += WindowRadioButtonRow;
        return WindowRadioButtonRow;
   
    def AddProgressbarToFrame(self, frameName):
        WindowProgressionProgressbarRow = WindowCreateProgressbar(self.__Istance);
        Progression = ProgressbarProgression(
            WindowProgressionProgressbarRow,
            self.AddLabelToFrame("0%", "Progressbar", TK.LEFT)
        );

        WindowProgressionProgressbarRow.pack(in_=self.__Frames[frameName], side=TK.LEFT);
        return Progression;

    def DisableAllButtons(self):
        for button in self.__ButtonsToDisable:
            button["state"] = "disabled";
        for rbutton in self.__RadioButtons:
            rbutton["state"] = "disabled";

    def Resize(self):
        self.__Istance.update_idletasks();
        self.__Istance.geometry(
            f"{
                self.__Istance.winfo_reqwidth()+WindowElementsDefaultPaddingWidth
            }x{
                self.__Istance.winfo_reqheight()+WindowElementsDefaultPaddingHeight
            }"
        );

    def Mainloop(self):
        self.Resize();
        self.__IsRunning = True;
        self.__Istance.mainloop();
   
    def Refresh(self):
        if self.__IsRunning and self.__Istance.winfo_exists():
            self.__Istance.update();
        else:
            sys.exit();

    def Destroy(self):
        print(f"{self.__Istance.title()} è stato Terminato!");
        messagebox.showinfo(self.__Istance.title(), "Le Operazioni sono state Concluse con successo!");
        self.__Istance.destroy();

    def Quit(self, message: str = ''):
        if message != '':
            messagebox.showerror(self.__Istance.title(), message);
        print(f"{self.__Istance.title()} è stato Terminato!");
        self.__IsRunning = False;
        self.__Istance.quit();
        sys.exit();